// (c) Copyright 2011-2012 Cloudera, Inc. All rights reserved.
define([
  'cloudera/cmf/wizard/express/ClusterCreationStep'
], function(ClusterCreationStep) {
  describe("ClusterCreationStep Tests", function() {
    var module, id = "clusterCreationStep";

    beforeEach(function() {
      $("<div>").attr("id", id).appendTo(document.body);
      jasmine.Ajax.useMock();
      clearAjaxRequests();
    });

    afterEach(function() {
      $("#" + id).remove();
    });

    it("should create and delete a cluster once", function() {
      var options = {
        id: id,
        createUrl: "dontcare",
        hosts: function() {
          return ["h1", "h2"];
        },
        version: function() {
          return "1.0";
        }
      };
      var module = new ClusterCreationStep(options);
      var callback = function() {};

      spyOn(module, "createCluster").andCallThrough();
      module.onEnter(callback, true);

      var request = mostRecentAjaxRequest();
      request.response({
        status: 200,
        responseText: JSON.stringify({
          message: "OK",
          data: {
            id: "C1",
            name: "Cluster"
          }
        })
      });
      expect(module.cluster().name).toEqual("Cluster");
      expect(module.cluster().id).toEqual("C1");
      expect(module.created).toBeTruthy();
      expect(module.createCluster.callCount).toEqual(1);

      module.onEnter(callback, true);
      // Expect createCluster was not called again.
      expect(module.createCluster.callCount).toEqual(1);

      spyOn(module, "deleteCluster").andCallThrough();
      module.onEnter(callback, false);

      request = mostRecentAjaxRequest();
      request.response({
        status: 200,
        responseText: JSON.stringify({
          message: "OK"
        })
      });
      expect(module.cluster().name).toBeUndefined();
      expect(module.cluster().id).toBeUndefined();
      expect(module.created).toBeFalsy();
      expect(module.deleteCluster.callCount).toEqual(1);

      module.onEnter(callback, true);
      // Expect deleteCluster was not called again.
      expect(module.deleteCluster.callCount).toEqual(1);
    });
  });
});
